/** 
 * Prepare for calling obj.property(...): return function corresponding to obj.property and make obj properly converted to Scriptable available as ScriptRuntime.lastStoredScriptable() for consumption as thisObj. The caller must call ScriptRuntime.lastStoredScriptable() immediately after calling this method.
 */
public static Callable getPropFunctionAndThis(Object obj,String property,Context cx){
  Scriptable thisObj=toObjectOrNull(cx,obj);
  if (thisObj == null) {
    throw undefCallError(obj,property);
  }
  Object value;
  for (; ; ) {
    value=ScriptableObject.getProperty(thisObj,property);
    if (value != Scriptable.NOT_FOUND) {
      break;
    }
    if (!(thisObj instanceof XMLObject)) {
      break;
    }
    XMLObject xmlObject=(XMLObject)thisObj;
    Scriptable extra=xmlObject.getExtraMethodSource(cx);
    if (extra == null) {
      break;
    }
    thisObj=extra;
  }
  if (!(value instanceof Callable)) {
    Object noSuchMethod=ScriptableObject.getProperty(thisObj,"__noSuchMethod__");
    if (noSuchMethod instanceof Callable)     value=new NoSuchMethodShim((Callable)noSuchMethod,property);
 else     throw notFunctionError(thisObj,value,property);
  }
  storeScriptable(cx,thisObj);
  return (Callable)value;
}
