package mining.commits;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Helper {
	public static String COMMIT_HASH = "\\s[a-z0-9]{40}";
	public static String commit_HASH = "[a-z0-9]{7}\\s";
	
	public static enum COMMIT_TYPE {TESTCASE, ADD, MODIFICATION};
	
	public static String TEST_FILER = "(test)|(unit)";
	public static String FIX_TESTFAILURE = "((fix)(.)*(test)(.)*(failure))|((test)(.)*(failure)(.)*(fix))";
	
	public static String TES_TFAILURE = "(test)(.)*(failure)";
	
	public static String BUG_TRACKING_NUMBER = "((\\s)|(-))(([0-9])+)";
	
	public static String BUG_FIX_TEST = String.format("((%s)(.)*(%s))|((%s)(.)*(%s))", FIX_TESTFAILURE, BUG_TRACKING_NUMBER, BUG_TRACKING_NUMBER, FIX_TESTFAILURE);
	
	public static String REG_CONF_FILE_PATH = "/Users/assist/dataset/TCM/resources/reg_keywords";
	public static String LOG_FILE_PATH = "/Users/assist/dataset/TCM/resources/commit-logs/aries.txt";
	
	
	public static String LOG_INPUT_FOLDER = "/Users/assist/dataset/TCM/resources/input_commit-logs";
	public static String HASH_OUTPUT_FOLDER = "/Users/assist/dataset/TCM/resources/output_commit_hashes";
	
	
	public static List<String> readFile(String path){
		Charset charset = Charset.forName("UTF-8");
		try {
			return Files.readAllLines(Paths.get(path), charset);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	
	public static boolean matchCommitHash(String text){
		String patter = COMMIT_HASH;
		Pattern r = Pattern.compile(patter);
		Matcher m = r.matcher(text);
		return m.find();
	}
	
	public static boolean matchPattern(String text, String reg){
		Pattern r = Pattern.compile(reg, Pattern.CASE_INSENSITIVE);
		Matcher m = r.matcher(text);
		return m.find();
	}
	
	
	private static String createIndividualRegFromFile(String filePath, COMMIT_TYPE type){
		List<String> lines = readFile(filePath);
		String reg = "";
		for (String line : lines){
			if (line.isEmpty())
				continue;
			String[] words = line.split(":");
			String key = words[0].split("\\s")[0].trim();
			String[] values = words[1].trim().split("-");
			if (key.toUpperCase().equals(type.toString().toUpperCase())){
				for (String value : values) {
					reg += "|" + "(" + value.trim() + ")";
				}
			}

		}

		return reg.substring(1);
	}
	
	
	public static String createRegToFilterCommits(COMMIT_TYPE filter) {
		String verb_reg = null;
		String test_reg = createIndividualRegFromFile(REG_CONF_FILE_PATH, COMMIT_TYPE.TESTCASE);
		
		switch (filter) {
			case ADD: 
				verb_reg = createIndividualRegFromFile(REG_CONF_FILE_PATH, COMMIT_TYPE.ADD);
				break;
			case MODIFICATION: 
				verb_reg = createIndividualRegFromFile(REG_CONF_FILE_PATH, COMMIT_TYPE.MODIFICATION);
				break;
			case TESTCASE:
				return test_reg;
		}
		
		String global_reg = "(" + "(" + verb_reg + ")" + "((.)*)" + "(" + test_reg + ")" + ")" + "|" + "(" + "(" + test_reg + ")" + "((.)*)" + "(" + verb_reg + ")" + ")";
		
		System.out.println(global_reg);
		
		return global_reg;
	}
	
	
	public static List<String> extractShortHashes(String filePath) {
		List<String> shortHashes = new ArrayList<String>();
		List<String> lines = readFile(filePath);
		for (String line : lines) {
			shortHashes.add(line.substring(0, 7));
		}
		return shortHashes;
	}
	
	public static String straightenList(List<String> lst) {
		String s = "";
		for (String str : lst) {
			s +=str + " ";
		}
		return s.trim();
	}
	
	public static int getNumberofCommittedFiles(List<Commit> commits){
		int total = 0;
		for (Commit c : commits) {
			total += c.getTotalFiles();
		}
		return total;
	}
	
	public static int getNumberofTestCommittedFiles(List<Commit> commits){
		int total = 0;
		for (Commit c : commits) {
			total += c.getnTestFiles();
		}
		return total;
	}
	
	public static int getNumberofCUTCommitedFiles(List<Commit> commits) {
		int total = 0;
		for (Commit c : commits){
			total += c.getnImplFiles();
		}
		return total;
	}
	
	
	/*
	 * Write commits to a file
	 */
	public static void writeCommitsToFile(String filePath, List<Commit> commits) throws IOException {
		File file = new File(filePath);
		file.createNewFile();
		FileWriter fWriter = new FileWriter(file.getAbsolutePath());
		BufferedWriter bWriter = new BufferedWriter(fWriter);
		
		for (Commit c : commits){
			bWriter.write(c.toString());
			bWriter.newLine();
			
			c.print();
		}
		
		bWriter.close();
	}
	
	/*
	 * Write string to a file
	 */
	public static void writeStringtoFile(String filePath, String text) throws IOException{
		File file = new File(filePath);
		file.createNewFile();
		FileWriter fWriter = new FileWriter(file.getAbsolutePath());
		BufferedWriter bWriter = new BufferedWriter(fWriter);
		bWriter.write(text);
		bWriter.close();
	}

}
