package td.brclassifier;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XMLReader {
	
	public static final String stackTraceRegex = "(\\s[^\\s]+Exception[\\s|\n]*)*(\\s+at.+[(]*.+java:[0-9]+[)]*)+";
	public static final String stackTraceRegex2 = "(\\s+at.+[(].+java:[0-9]+[)])+";
	
	public static List<Bug> readXMLFile (String xmlFile) throws SAXException, IOException {
		
		try {
			
			List<Bug> bugList = new ArrayList<Bug>();
			
			File fXmlFile = new File(xmlFile);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder;
			dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(fXmlFile);
			
			doc.getDocumentElement().normalize();
			
			//System.out.println("Root element: " + doc.getDocumentElement().getNodeName());
			
			NodeList nList = doc.getElementsByTagName("bug");
			
			System.out.println(nList.getLength());
			
			for (int i=0; i< nList.getLength(); i++) {
				Node node = nList.item(i);
				
				//System.out.println("Current element: " + node.getNodeName());
				
				Node bugInfoNode = node.getChildNodes().item(1); // buginformation node
				
				//System.out.println(bugInfoNode.getNodeName());
				
				String bugID = node.getAttributes().getNamedItem("id").getTextContent(); // bug report's id
				
				System.out.println("Bug ID.: " + bugID);
				
				String summary = bugInfoNode.getChildNodes().item(1).getTextContent(); // bug report summary content
				String description = bugInfoNode.getChildNodes().item(3).getTextContent(); // bug report description field
				
				System.out.println("Summary: " + summary);
				System.out.println("Description: " + description);
				
				System.out.println("\n");
				
				Bug bug = new Bug(bugID, summary, description);
				bugList.add(bug);
			}
			
			return bugList;
			
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
		
	}
	
	public static List<String> extractStackTrace(String input) {
		Pattern r = Pattern.compile(stackTraceRegex2);
		Matcher m = r.matcher(input);
		List<String> foundStackTraces = new ArrayList<String>();
		while (m.find()){
			foundStackTraces.add(m.group());
		}
		foundStackTraces.add(m.replaceAll(""));
		return foundStackTraces;
	}

	public static void main(String[] args) throws SAXException, IOException {
		// TODO Auto-generated method stub
		
		//List<Bug> bList = readXMLFile("/Users/assist/ase2016/code/RTP/dbbugreports/AspectJBugRepository.xml");
		//System.out.println("The total number of bug reports: " + bList.size());
		
		//readXMLFile("/Users/assist/ase2016/code/RTP/dbbugreports/EclipseBugRepository.xml");
		
		String testString = "Noticed these in my .log java.lang.NullPointerException at org.eclipse.ant.internal.ui.model.AntProjectNodeProxy.getOffset (AntProjectNodeProxy.java:209) at org.eclipse.ant.internal.ui.model.AntElementNode.toString (AntElementNode.java:300) at org.eclipse.ant.internal.ui.views.AntView$3.compare (AntView.java:308) at org.eclipse.jface.viewers.ViewerSorter$1.compare (ViewerSorter.java:177) at java.util.Arrays.mergeSort(Arrays.java:1292) at java.util.Arrays.mergeSort(Arrays.java:1304) at java.util.Arrays.sort(Arrays.java:1233) at org.eclipse.jface.viewers.ViewerSorter.sort(ViewerSorter.java:175) at org.eclipse.jface.viewers.StructuredViewer.getSortedChildren (StructuredViewer.java:782) at org.eclipse.jface.viewers.AbstractTreeViewer.updateChildren (AbstractTreeViewer.java:1517) at org.eclipse.jface.viewers.AbstractTreeViewer.internalRefreshStruct (AbstractTreeViewer.java:1094) at org.eclipse.jface.viewers.AbstractTreeViewer.internalRefresh (AbstractTreeViewer.java:1071) at org.eclipse.jface.viewers.AbstractTreeViewer.internalRefresh (AbstractTreeViewer.java:1032) at org.eclipse.jface.viewers.AbstractTreeViewer.internalRefresh (AbstractTreeViewer.java:1019) at org.eclipse.jface.viewers.StructuredViewer$7.run (StructuredViewer.java:1127) at org.eclipse.jface.viewers.StructuredViewer.preservingSelection (StructuredViewer.java:1064) at org.eclipse.jface.viewers.StructuredViewer.refresh (StructuredViewer.java:1125) at org.eclipse.jface.viewers.StructuredViewer.refresh (StructuredViewer.java:1084) at org.eclipse.ant.internal.ui.views.AntView.addProject (AntView.java:450) at org.eclipse.ant.internal.ui.views.actions.SearchForBuildFilesAction$2.run (SearchForBuildFilesAction.java:72) at org.eclipse.swt.widgets.RunnableLock.run(RunnableLock.java:35) at org.eclipse.swt.widgets.Synchronizer.runAsyncMessages (Synchronizer.java:118) at org.eclipse.swt.widgets.Display.runAsyncMessages(Display.java:2771) at org.eclipse.swt.widgets.Display.readAndDispatch(Display.java:2451) at org.eclipse.ui.internal.Workbench.runEventLoop(Workbench.java:1443) at org.eclipse.ui.internal.Workbench.runUI(Workbench.java:1414) at org.eclipse.ui.internal.Workbench.createAndRunWorkbench (Workbench.java:271) at org.eclipse.ui.PlatformUI.createAndRunWorkbench(PlatformUI.java:144) at org.eclipse.ui.internal.ide.IDEApplication.run (IDEApplication.java:102) at org.eclipse.core.internal.runtime.PlatformActivator$1.run (PlatformActivator.java:335) at org.eclipse.core.runtime.adaptor.EclipseStarter.run (EclipseStarter.java:273) at org.eclipse.core.runtime.adaptor.EclipseStarter.run (EclipseStarter.java:129) at sun.reflect.NativeMethodAccessorImpl.invoke0(Native Method) at sun.reflect.NativeMethodAccessorImpl.invoke (NativeMethodAccessorImpl.java:85) at sun.reflect.NativeMethodAccessorImpl.invoke (NativeMethodAccessorImpl.java:58) at sun.reflect.DelegatingMethodAccessorImpl.invoke (DelegatingMethodAccessorImpl.java:60) at java.lang.reflect.Method.invoke(Method.java:391) at org.eclipse.core.launcher.Main.basicRun(Main.java:185) at org.eclipse.core.launcher.Main.run(Main.java:684) at org.eclipse.core.launcher.Main.main(Main.java:668)";
		String testString2 = "Exception in thread \"main\" java.lang.Error: Unresolved compilation problem:\n" + 
								"This method must return a result of type String\n" + 

										"at td.brclassifier.XMLReader.extractStackTrace(XMLReader.java:80)\n"+
										"at td.brclassifier.XMLReader.main(XMLReader.java:100)";
		String testString3 = "there is no stack trace";
		
		System.out.println(extractStackTrace(testString3).get(1));
	}

}
