package td.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import rtp.utility.FileSystem;
import rtp.extractor.ClassExtractor;
import rtp.extractor.PreProcessor;;
public class BugReportConverter2 {
	
	private static String CODE_DIR = "/Users/assist/dmz/ir-dataset/code/";
	private static String VERSION = "pre-fix";
	
	/**
	 * Read groundtruth and return as a list of 
	 * fixed files
	 * @param pathToFixedFile
	 * @return
	 * @throws IOException
	 */
	public static List<String> readFixedFiles(String pathToFixedFile) throws IOException {
		String[] fixedFiles = FileSystem.readFile(pathToFixedFile).split("\n");
		List<String> temp = new ArrayList<String>();
		for (String f : fixedFiles) {
			if (f != null && ! f.trim().isEmpty() && ! f.startsWith(".")) {
				String tmp = "";
				if (f.contains("$")) {
					tmp = f.replaceAll("/", ".").split("\\$")[0].trim();
					System.out.println("Found: " + f);
				} else {
					tmp = f.replaceAll("/", ".").split("_")[0].trim();
				}
				
				if (tmp.contains("java.")) {
					tmp = tmp.split("java.")[1] + ".java";
				} else if (tmp.contains("test.")) {
					tmp = tmp.split("test.")[1] + ".java";
				} else if (tmp.contains("src.")) {
					tmp = tmp.split("src.")[1] + ".java";
				} else {
//					System.out.println("[error] Something wrong with the ground truth: " + pathToFixedFile + " " + f);
//					System.exit(0);
					tmp += ".java";
				}
				temp.add(tmp);
			}
		}
		return new ArrayList<String>(new HashSet<String>(temp));
	}
	
	/**
	 * In this case, fixed files are the whole java 
	 * files of the source code
	 * @param projectID
	 * @param bugID
	 * @return
	 * @throws IOException
	 */
	public static List<String> readFixedFiles2(String projectID, String bugID) throws IOException {
		
		FileSystem fs = new FileSystem();
		String codePath = CODE_DIR + projectID + "/" + bugID + "/" + VERSION;
		
		if (projectID.contains("ant")){
			codePath += "/" + "src";
		} else if (projectID.contains("lucene")) {
			//codePath += "/" + "lucene" + "/" + "src";
			codePath += "/" + "lucene";
		} else if (projectID.contains("rhino")) {
			//codePath += "/" + "src";
			codePath += "/src";
		} else if (projectID.contains("aspectj")) {
			codePath += "/" + "org.aspectj/modules/weaver";
		}
		
		
		List<String> temp = fs.getAllJavaFiles(codePath, codePath.length());
		
		List<String> correctedFQNs = new ArrayList<String>();
		for (String s : temp) {
			s = s.replaceAll("/", ".");
			if (projectID.contains("ant")) {
				if (s.contains("org.apache")) {
					s = "org.apache" + s.split("org.apache")[1].trim();
					correctedFQNs.add(s);
				}
			} else if (projectID.contains("rhino")) {
				if (s.contains("org.mozilla")) {
					s = "org.mozilla" + s.split("org.mozilla")[1].trim();
					correctedFQNs.add(s);
				}
			}else if (projectID.contains("lucene")) {
				if (s.contains("org.apache")) {
					s = "org.apache" + s.split("org.apache")[1].trim();
					correctedFQNs.add(s);
				}
			} else if (projectID.contains("aspectj")) {
				if (s.contains("pre-fix.org.aspectj.modules.weaver.testsrc.")) {
					s = s.split("pre-fix.org.aspectj.modules.weaver.testsrc.")[1].trim();
					correctedFQNs.add(s);
				}
				
//				if (s.contains("org.aspectj.")) {
//					s = "org.aspectj." + s.split("org.aspectj.")[1].trim();
//					if (s.endsWith(".java")){
//						correctedFQNs.add(s);
//					} else {
//						System.out.println(s);
//					}
//				}
			}
			
		}
		
		return new ArrayList<String>(new HashSet<String>(correctedFQNs));
	}
	/**
	 * (1) get bug report contents
	 * (2) get fixed files: either from ground truth or the whole source code files
	 * (3) compose the bug report in xml format
	 * @param projectID
	 * @param bugID
	 * @param pathToBR
	 * @param pathToFixedFiles
	 * @param outputXMLFile
	 * @throws IOException
	 */
	public static void composeBugReportXML(String projectID, String bugID, String pathToBR, String pathToFixedFiles, String outputXMLFile) throws IOException{
		String brContent = FileSystem.readFile(pathToBR);
		String[] parts = brContent.split(":::");
		System.out.println(parts.length);
		
		String bugId = pathToBR.split("/")[pathToBR.split("/").length - 1];
		
		String title = parts[1].trim();
		String description = parts[2].trim();
		
		List<String> fixedFiles = readFixedFiles2(projectID, bugID); // fixed files are all java files	
		
		if (fixedFiles == null || fixedFiles.isEmpty() || fixedFiles.size() == 0) {
			System.out.println("Something wrong");
			System.exit(0);
		}
	
		FileWriter fw = new FileWriter(outputXMLFile);
		BufferedWriter bw = new BufferedWriter(fw);
		bw.write("<bugrepository name=\"" + projectID + "\"" + ">");
		bw.newLine();
		
		bw.write("\t<bug id=\"" + bugId.split("-")[1].trim() +  "\" opendate=\"\" fixdate=\"\">");
		bw.newLine();
			
		bw.write("\t\t<buginformation>");
		bw.newLine();
		
		bw.write("\t\t\t<summary>");
		bw.newLine();
		
		bw.write("\t\t\t\t" + PreProcessor.process(title));
		bw.newLine();
		
		bw.write("\t\t\t</summary>");
		bw.newLine();
		
		bw.write("\t\t\t<description>");
		bw.newLine();
		
		bw.write("\t\t\t\t" + PreProcessor.process(description));
		bw.newLine();
		
		bw.write("\t\t\t</description>");
		bw.newLine();
		
		bw.write("\t\t</buginformation>");
		bw.newLine();
		
		bw.write("\t\t<fixedFiles>");
		bw.newLine();
		
		for (String f : fixedFiles) {
			bw.write("\t\t\t<file>");
			
			if (f.contains("/")){
				bw.write(f.split(VERSION + "/")[1].replace("/", "."));
			} else {
				bw.write(f);
			}
			
			bw.write("</file>");
			bw.newLine();	
			
		}
		
		bw.write("\t\t</fixedFiles>");
		bw.newLine();
		
		bw.write("\t</bug>");
		bw.newLine();
		
		bw.write("</bugrepository>");
		bw.newLine();
		
		bw.close();	
		fw.close();
	}
	
	public static void mergeBugReports(String projectName, List<String> list, String xmlFilePath) throws IOException {
		FileWriter fw = new FileWriter(xmlFilePath);
		BufferedWriter bw = new BufferedWriter(fw);
		bw.write("<bugrepository name=" + "\"" + projectName + "\"" + ">");
		bw.newLine();
		
		for (String elem : list) {
			bw.write(elem);
			bw.newLine();
			bw.newLine();
		}
		
		bw.write("</bugrepository>");
		bw.newLine();
		
		bw.close();	
		fw.close();
	}
	
	public static void convertBugReports(String bugFolderPath, String fixedFilesFolderPath, String outputPath) throws IOException {
		
		if (! new File(outputPath).exists()) {
			new File(outputPath).mkdir();
		}
		
		File directory = new File(bugFolderPath);
		File[] files = directory.listFiles();
		
		for (File f : files) {
			if (f.isDirectory() && f.getName().equals("aspectj")){ //&& f.getName().toLowerCase().contains("aspectj")) {
				String projectName = f.getName();
				System.out.println(projectName);
				
				File dir = new File(outputPath + "/" + projectName);
				dir.mkdir(); // create a corresponding output folder
				
				File[] bugReports = f.listFiles();
				for (File br : bugReports) {
					String bugId = br.getName();
					//System.out.println(bugId);
					
					String pathToBR = br.getAbsolutePath();
					System.out.println(pathToBR);
					
					String pathToFixedFiles = fixedFilesFolderPath + "/" + projectName + "/" + bugId + ".txt";
					System.out.println(pathToFixedFiles);
					
					String pathToOutputFile = dir.getAbsolutePath() + "/" + bugId + ".xml";
					System.out.println(pathToOutputFile);
					
					composeBugReportXML(projectName, bugId, pathToBR, pathToFixedFiles, pathToOutputFile);
					
					System.exit(0);
				}
			}
		}
	}
	
	/**
	 * 
	 * @param bugDirPath
	 * @param outDir
	 * @throws IOException
	 */
	public static void createBugRepository(String bugDirPath, String outDir) throws IOException {
		File bugDir = new File(bugDirPath);
		File[] projects = bugDir.listFiles();
		
		for (File p : projects) {
			List<String> elems = new ArrayList<String>();
			if (p.isDirectory()) {
				System.out.println("Working on " + p.getName());
				File[] xmlFiles = p.listFiles();
				for (File xmlFile : xmlFiles) {
					String content = FileSystem.readFile(xmlFile.getAbsolutePath()).trim();
					elems.add(content);
				}
				mergeBugReports(p.getName(), elems, outDir + "/" + p.getName() + ".xml");
			}
		}
		
		System.out.println("done");
	}
	
	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		String br = "/Users/assist/dmz/ir-dataset/bugs/bug-reports";
		String fixed = "/Users/assist/dmz/ir-dataset/ground_truth";
		String outputFile = "/Users/assist/dmz/ir-dataset/bugs-xml-repository2";
		String tempDir = "/Users/assist/dmz/ir-dataset/bugs-xml-repository";
		
		String outputFile2 = "/Users/assist/dmz/ir-dataset/bugs-xml-repository3";
		String tempDir2 = "/Users/assist/dmz/ir-dataset/bugs-xml-repository-temp";
		
		convertBugReports(br, fixed, outputFile2);
		
		//createBugRepository(outputFile2, tempDir2);
	}

}
