package td.utils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Comparator;

public class MetricsCalculator4BL {
	
	public static Map<String, Integer> getResults(String inputResultFile) throws IOException {
		HashMap<String, Integer> ranks = new HashMap<String, Integer>();
		
		List<String> lines = FileHandler.readAllLines(inputResultFile);
		
		for (String line : lines) {
			String location = line.split(",")[1];
			int rank = Integer.parseInt(line.split(",")[2]) + 1;
			ranks.put(location, rank);
		}
		
		return ranks;
	}
	
	public static Map<String, Integer> sortByComparator(Map<String, Integer> map) {
		List<Map.Entry<String, Integer>> list = new LinkedList<Map.Entry<String, Integer>>(map.entrySet());
		
		Collections.sort(list, new Comparator<Map.Entry<String, Integer>>() {
			public int compare(Map.Entry<String, Integer> opt1, Map.Entry<String, Integer> opt2) {
				return (opt1.getValue().compareTo(opt2.getValue()));
			}
		});
		
		Map<String, Integer> sortedMap = new LinkedHashMap<String, Integer>();
		
		for (Iterator<Map.Entry<String, Integer>> it = list.iterator(); it.hasNext();) {
			Map.Entry<String, Integer> entry  = it.next();
			sortedMap.put(entry.getKey(), entry.getValue());
		}
		
		return sortedMap;
	}
	
	public static void printMap(Map<String, Integer> map) {
		for (Map.Entry<String, Integer> entry : map.entrySet()) {
			System.out.println(entry.getKey() + " : " + entry.getValue());
		}
	}
	
	public static String printMapToString(Map<String, Integer> map) {
		String result = "";
		for (Map.Entry<String, Integer> entry : map.entrySet()) {
			//System.out.println(entry.getKey() + " : " + entry.getValue());
			result += entry.getKey() + " : " + entry.getValue() + "\n";
		}
		return result;
	}
	
	public static Metrics calMetrics(String qName, Map<String, Integer> sortedMap) {
		Metrics metric = new Metrics(qName, sortedMap);
		return metric;
	}

	public static HashMap<String, Map<String, Integer>> readAllRanks(String rankDir) throws IOException {

		File[] files = new File(rankDir).listFiles();
		
		//remember to remove this code: it is to filter out those queries in slicing set
//		String filter_file = "/Users/assist/dmz/gen-data/sIR_and_slicing/out/rhino_found_queries_set.txt";
//		String filtered_file_names = FileHandler.readAllLines(filter_file).toString();
//		System.out.println(filtered_file_names);
		//System.exit(0);
		// end of removing code later

		HashMap<String, Map<String, Integer>> ranks = new HashMap<String, Map<String, Integer>>();

		for (File file : files) {
			if (file.isFile()) {
				String filePath = file.getAbsolutePath();
				String name = file.getName().split("\\.")[0];
				
//				if (! filtered_file_names.contains(name)) {
//					System.out.println(name);
//					continue;
//				}
				
				
				Map<String, Integer> sortedMap = sortByComparator(getResults(filePath));

				//System.out.println(name);
				//printMap(sortedMap);
				//System.out.println("\n");
				ranks.put(name, sortedMap);
			}
		}
		
		return ranks;
	}
	
	public static Metrics calOveralMetrics (HashMap<String, Metrics> metricMap) {
		double MRR = 0, MPA = 0;
		int top1 = 0, top5 = 0, top10 = 0;
		int totalQueries = metricMap.size();
		for (Metrics metric : metricMap.values()) {
			MRR += metric.getMRR();
			MPA += metric.getMP();
			top1 += metric.getTop1();
			top5 += metric.getTop5();
			top10 += metric.getTop10();
		}
		
		Metrics overallMetrics = new Metrics();
		overallMetrics.setMP((double ) MPA/totalQueries);
		overallMetrics.setMRR((double) MRR/totalQueries);
		overallMetrics.setTop1(top1);
		overallMetrics.setTop5(top5);
		overallMetrics.setTop10(top10);
		overallMetrics.setQueryName("Overall Metrics");
		
		return overallMetrics;
	}
	
	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		
		String[] projects = {"ant53", "aspectj", "rhino", "lucene37"};

		//String resultsDir = "/Users/assist/dmz/gen-data/buglocator_and_slicing/" + projects[2];
		
		String resultsDir = "/Users/assist/dmz/gen-data/buglocator_and_cv/ant53";
		
		HashMap<String, Metrics> metricMap = new HashMap<String, Metrics>();
		
		HashMap<String, Map<String, Integer>> ranks = readAllRanks(resultsDir);
		Iterator<Entry<String, Map<String, Integer>>> it = ranks.entrySet().iterator();
		
		int count = 0;
		while (it.hasNext()) {
			System.out.println("[" + ++count + "]");
			
			Entry<String, Map<String, Integer>> pair = it.next();
			
			String queryName = pair.getKey();
			System.out.println(">>>Rankings of the query: " + queryName);
			
			Map<String, Integer> sortedRanks = pair.getValue();
			printMap(sortedRanks);
			
			Metrics metric = calMetrics(queryName, sortedRanks);
			metric.print();
			
			System.out.println("\n");
			
			metricMap.put(queryName, metric);
		}
		
		System.out.println("================================");
		
		Metrics finalMetric = calOveralMetrics(metricMap);
		finalMetric.print();
	}

}
