package td.utils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MetricsCalculator4IR {
	
	public static Map<String, Integer> getFoundRanks(String resultFile, String ground_truthFile) throws IOException {
		
		List<String> ranks = getRanks(resultFile);
		
		//List<String> ranks = getRanks4BL(resultFile);
		
		List<String> locations = getGroundTruth(ground_truthFile); // at class level
		
		Set<String> uniqueLocations = new HashSet<>();
		uniqueLocations.addAll(locations);
		
		HashMap<String, Integer> rankedFiles = new HashMap<String, Integer> ();
		
		for (String l : uniqueLocations) {
			boolean found = false;
			
			for (int i = 0; i<ranks.size(); i++){
				String r = ranks.get(i);
				if (l.endsWith(r) || r.endsWith(l)) {
					rankedFiles.put(l, i+1);
					found = true;
					break;
				}
			}
			
			if (! found) {
				System.out.println(l + " is not found");
			}
			
		}
		
		return MetricsCalculator4BL.sortByComparator(rankedFiles);
	}
	
	public static List<String> calProjectMetrics(String projectResultDir, String groundTruthDir, String outputDir) throws IOException{
		File[] files = new File(projectResultDir).listFiles();
		HashMap<String, Metrics> metricMap = new HashMap<String, Metrics>();
		
		List<String> validQueries = new ArrayList<String>();
		
		List<String> outputResults = new ArrayList<String>();
		
		int count = 0;
		for (File f : files) {
			if (f.isFile() && ! f.getName().startsWith(".")) {
				String id = f.getName();
				if (! id.contains(".txt")){
					id += ".txt";
				}
				File groundTruthFile = new File(groundTruthDir + "/" + id);
				String groudTruthPath = "";
				
				if (groundTruthFile.exists()) {
					groudTruthPath = groundTruthFile.getAbsolutePath();
				} else {
					System.out.println("Error: ground truth file does not exist :" + id);
					System.exit(0);
				}
				
				Map<String, Integer> rankedFiles = getFoundRanks(f.getAbsolutePath(), groudTruthPath);
				if (! rankedFiles.isEmpty()) {
					
					System.out.println(++count + ". " + f.getName());
					
					outputResults.add(Integer.toString(count) + ". " + f.getName());
					
					Metrics metric = new Metrics(id, rankedFiles);
					metricMap.put(id, metric);
					MetricsCalculator4BL.printMap(rankedFiles);
					
					outputResults.add(MetricsCalculator4BL.printMapToString(rankedFiles));
					
					metric.print();
					outputResults.add(metric.printToString());
					
					System.out.println("-------------------------");
					
					validQueries.add(f.getName());
				}
			}	
		}
		
		Metrics finalMetric = MetricsCalculator4BL.calOveralMetrics(metricMap);
		finalMetric.print();
		
		outputResults.add(finalMetric.printToString());
		
		FileHandler.writeListToFile(outputResults, outputDir);
		
		return validQueries;
	
	}
	
	public static List<String> getGroundTruth(String ground_truthFile) throws IOException {
		List<String> locations = new ArrayList<String>();
		for (String l : FileHandler.readAllLines(ground_truthFile)) {
			String className = l.split("_")[0];
			
			if (className.contains("/")) {
				className = className.replaceAll("/", ".");
			}
			
			String[] classNameParts = className.split("\\.");
			String realName = classNameParts[classNameParts.length-1];
			if (! Character.isUpperCase(realName.charAt(0))) {
				System.out.println("Class full name: " + className);
				System.out.println("Class short name: " + realName);
				System.out.println(realName + "does not start with an upper case!!!");
				System.out.println("Something wrong. Check: " + ground_truthFile);
				System.exit(0);
			}
			
			if (className.contains("$")) {
				className = className.split("\\$")[0];
			}
			
			if (! className.endsWith(".java")) {
				className += ".java";
			}
			
			if (!className.startsWith(".")) {
				className = "." + className;
			}
			
			locations.add(className);
		}
		return locations;
	}
	
	public static List<String> getRanks(String resultFile) throws IOException {
		List<String> ranks = new ArrayList<String> ();
		try {
			for (String r : FileHandler.readAllLines(resultFile)) {
				String rName = r.split(",")[1].trim();// position = 0 for Tarantula and 2 for normal IR
				if (! rName.endsWith(".java")) {
					rName += ".java";
				}
				ranks.add(rName);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(resultFile);
		}
		return ranks;
	}
	
	public static List<String> getRanks4BL(String resultFile) throws IOException {
		List<String> ranks = new ArrayList<String> ();
		try {
			for (String r : FileHandler.readAllLines(resultFile)) {
//				String rName = r.split(",")[1].trim();// position = 0 for Tarantula and 2 for normal IR
//				if (! rName.endsWith(".java")) {
//					rName += ".java";
//				}
//				ranks.add(rName);
				ranks.add(r);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(resultFile);
		}
		return ranks;
	}
	
	public static void printList(List<String> list) {
		for (String l : list) {
			System.out.println(l);
		}
	}
	
	//0965 818 646 {Doan Thi Thi Hien}
	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		
//		String projectResultPath = "/Users/assist/dmz/gen-data/ir/class-ir/lucene37/cv_2/baseline1/results";
//		String groundTruthPath = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/lucene37";
		
		//String projectResultPath = "/Users/assist/dmz/gen-data/sIR_and_slicing/out/rhino";
		
		//String validQueriesPath = "/Users/assist/dmz/gen-data/sIR_and_slicing2/foundqueries/rhino.txt";
		
		//String groundTruthPath = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/rhino";
		
		
		
		//List<String> foundQueries = calProjectMetrics(projectResultPath, groundTruthPath);
		
		//FileHandler.writeListToFile(foundQueries, validQueriesPath);
		
//		String projectResultPath = "/Users/assist/dmz/gen-data/Spectrum/IRwSP_Tarantula_Variations/class/projects/rhino";// + projects[i];
//		String groundTruthPath = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/rhino";// + projects[i];
//		String outputDir = "/Users/assist/dmz/gen-data/spectrum/IRwSP_Tarantula_Variations/class/results/rhino";
//		
//		for (int i=0; i<11 ; i++){
//			calProjectMetrics(projectResultPath + "/" + "comb_" + i, groundTruthPath, outputDir + "/" + "comb_" + i + ".txt");
//		}
		
		//calProjectMetrics("/Users/assist/Desktop/IRSPcombined/backup/buglocator-slicing-cv/class/lucene37", "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth_corrected_4_class/lucene37", "/Users/assist/Desktop/IRSPcombined/backup/buglocator-slicing-cv/class/lucene37.txt");
		
		//calProjectMetrics("/Users/assist/Desktop/IRSPcombined/backup/buglocator-slicing-cv/class/rhino", "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/rhino", "/Users/assist/Desktop/IRSPcombined/backup/buglocator-slicing-cv/class/rhino.txt");
		
		//calProjectMetrics("/Users/assist/dmz/gen-data/ir/class-ir/rhino/all/results/post-fix", "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/rhino", "/Users/assist/Desktop/rhino-ir-post.txt");
		calProjectMetrics("/Users/assist/dmz/filter-code/coverage-buglocator-output/aspectj", "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/aspectj", "/Users/assist/dmz/filter-code/scores/aspectj.txt");
	}

}
