package td.utils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MetricsCalculator4IRMethod {
	
	/**
	 * get ranks of the ground truth locations (methods)
	 * then calculate metrics of those found ranked methods
	 * @param resultFile
	 * @param ground_truthFile
	 * @return
	 * @throws IOException
	 */
	public static Map<String, Integer> getFoundRanks(String resultFile, String ground_truthFile) throws IOException {
		
		//List<String> ranks = getRanks4BL(resultFile);
		
		List<String> ranks = getRanks(resultFile);
		
		List<String> locations = getGroundTruth(ground_truthFile);
		
		Set<String> uniqueLocations = new HashSet<>();
		uniqueLocations.addAll(locations);
		
		HashMap<String, Integer> rankedMethods = new HashMap<String, Integer> ();
		
		for (String l : uniqueLocations) {
			
			String l4BL = l;
		
			if (! l4BL.endsWith(".java")) {
				l4BL += ".java";
			} 
			
			boolean found = false;
			for (int i = 0; i<ranks.size(); i++){
				String r = ranks.get(i);
				
				if (! r.endsWith(".java")) {
					r = r + ".java";
				}
				
				if (l4BL.endsWith(r) || r.endsWith(l4BL)) {
				//if (l.contains(r) || r.contains(l)) {
					rankedMethods.put(l, i+1);
					found = true;
					break;
				}
			}
			
			if (! found) {
				System.out.println(ground_truthFile + ">>>" + l4BL + " is not found");
			}
		}
		
		return MetricsCalculator4BL.sortByComparator(rankedMethods);
	}
	
	/**
	 * Calculate metrics for the whole project
	 * @param projectResultDir
	 * @param groundTruthDir
	 * @throws IOException
	 */
	public static List<String> calProjectMetrics(String projectResultDir, String groundTruthDir, String outputDir) throws IOException{
		File[] files = new File(projectResultDir).listFiles();
		HashMap<String, Metrics> metricMap = new HashMap<String, Metrics>();
		
		List<String> unFoundQueries = new ArrayList<String>();
		
		List<String> foundQueries = new ArrayList<String>();
		
		List<String> outputResults = new ArrayList<String>();
		
		int count = 0;
		for (File f : files) {
			if (f.isFile() && ! f.getName().startsWith(".")) {
				String id = f.getName();
				if (! id.contains(".txt")){
					id += ".txt";
				}
				File groundTruthFile = new File(groundTruthDir + "/" + id);
				String groudTruthPath = "";
				if (groundTruthFile.exists() && (! groundTruthFile.isDirectory())) {
					groudTruthPath = groundTruthFile.getAbsolutePath();
				} else {
					System.out.println("Error: ground truth file does not exist : " + id);
					System.exit(0);
				}
				
				Map<String, Integer> rankedFiles = getFoundRanks(f.getAbsolutePath(), groudTruthPath);
				if (! rankedFiles.isEmpty()) {
					System.out.println(++count + ". " + f.getName());
					outputResults.add(Integer.toString(count) + ". " + f.getName());
					Metrics metric = new Metrics(id, rankedFiles);
					metricMap.put(id, metric);
					MetricsCalculator4BL.printMap(rankedFiles);
					outputResults.add(MetricsCalculator4BL.printMapToString(rankedFiles));
					metric.print();
					outputResults.add(metric.printToString());
					System.out.println("-------------------------");
					foundQueries.add(f.getName());
				} else {
					//System.out.println(++count);
					//System.out.println("ground truth is not found in: " + f);
					//System.out.println("--------------------------");
					unFoundQueries.add(id + ":" + f.getAbsolutePath());
				}
			}
		}
		
		Metrics finalMetric = MetricsCalculator4BL.calOveralMetrics(metricMap);
		
		finalMetric.print();
		
		outputResults.add(finalMetric.printToString());
		
		if (! new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}
		
		
		FileHandler.writeListToFile(outputResults, outputDir + "/" + "calcuated_results.txt");
		
		//printList(unFoundQueries);
		
		return foundQueries;
	
	}
	/**
	 * Get ground truth at method levels
	 * @param ground_truthFile
	 * @return
	 * @throws IOException
	 */
	public static List<String> getGroundTruth(String ground_truthFile) throws IOException {
		List<String> locations = new ArrayList<String>();
		for (String l : FileHandler.readAllLines(ground_truthFile)) {
			if (! l.contains("_")) {
				System.out.println("[Error] Something wrong with extracting ground truth at: " + l);
				System.exit(0);
			}
			
			String className = l.split("_")[0];
			if (className.contains("$")) {
				className = className.split("\\$")[0];
			}
			
			String methodName = l.substring(className.length() + 1);
			
			if (methodName.contains("-")){
				methodName = methodName.split("-")[0];
			} else if (methodName.contains("_")) {
				methodName = methodName.substring(0, methodName.length() - methodName.split("_")[methodName.split("_").length-1].length() - 1);
			} else {
				System.out.println("[Error] Something wrong with extracting ground truth at: " + l);
				System.exit(0);
			}
				
			
			String fulMethodName = className + "." + methodName;
			fulMethodName = fulMethodName.replace("/", ".");
			
			locations.add(fulMethodName);
		}
		return locations;
	}
	
	/**
	 * given the raw result file, extract the method names in their ranking order
	 * @param resultFile
	 * @return
	 * @throws IOException
	 */
	public static List<String> getRanks(String resultFile) throws IOException {
		List<String> ranks = new ArrayList<String> ();
		try {
			for (String r : FileHandler.readAllLines(resultFile)) {
				String rName = r.split(" ")[2].trim(); // position = 2 for IR, and = 0 for combined
				ranks.add(rName);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(resultFile);
		}
		
		return ranks;
	}
	
	public static List<String> getRanks4BL(String resultFile) throws IOException {
		List<String> ranks = new ArrayList<String> ();
		try {
			for (String r : FileHandler.readAllLines(resultFile)) {
				String rName = r.split(",")[1].trim(); // position = 2 for IR, and = 0 for combined
				ranks.add(rName);
				//ranks.add(r);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(resultFile);
		}
		
		return ranks;
	}
	
	
	public static void printList(List<String> list) {
		for (String l : list) {
			System.out.println(l);
		}
	}
	
	//0965 818 646 {Doan Thi Thi Hien}
	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		
//		String projectResultPath = "/Users/assist/dmz/gen-data/ir/class-ir/lucene37/cv_2/baseline1/results";
//		String groundTruthPath = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/lucene37";
		
		String[] projects = new String[] {"ant53", "lucene37", "rhino"};
		
		//int i= 2;
		
//		String projectResultPath = "/Users/assist/dmz/gen-data/ir_method_and_slicing2/out/lucene37";// + projects[i];
//		String groundTruthPath = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth_corrected_4_method/lucene37";// + projects[i];
		
		
		//String validQueriesPath = "/Users/assist/dmz/gen-data/sIR_method_and_slicing2/foundqueries/" + projects[i] + ".txt";
		
//		String projectResultPath = "/Users/assist/dmz/gen-data/Spectrum/IRwSP_Tarantula_Variations/method/projects/rhino";// + projects[i];
//		String groundTruthPath = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth/rhino";// + projects[i];
//		String outputDir = "/Users/assist/dmz/gen-data/Spectrum/IRwSP_Tarantula_Variations/method/results/rhino";
//		
//		for (int i=0; i<11 ; i++){
//			calProjectMetrics(projectResultPath + "/" + "comb_" + i, groundTruthPath, outputDir + "/" + "comb_" + i + ".txt");
//		}
		
		//List<String> foundQuries = calProjectMetrics(projectResultPath, groundTruthPath, outputDir);
		
		//FileHandler.writeListToFile(foundQuries, validQueriesPath);
		
		//calProjectMetrics("/Users/assist/dmz/filter-code/coverage-method-buglocator-output/aspectj", "/Users/assist/ase2016/code/RTP/DLo/amldata/ground_truth/aspectj", "/Users/assist/dmz/filter-code/coverage-buglocator-results/aspectj");
		//calProjectMetrics("/Users/assist/Desktop/IRSPcombined/backup/buglocator-slicing-cv/method/rhino", "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth_corrected_4_method/rhino", "/Users/assist/Desktop/IRSPcombined/backup/buglocator-slicing-cv/method/rhino_corrected.txt");
		
		calProjectMetrics("/Users/assist/dmz/gen-data/ir/method-ir/lucene37/cv/union/results/pre-fix", "/Users/assist/ase2016/code/RTP/DLo/amldata/ground_truth/lucene", "/Users/assist/dmz/gen-data/ir/method-ir/lucene37/cv/union/results/pre-fix-result");
	}

}
