package td.utils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import rtp.utility.FileSystem;

public class Misc {
	
	public static void extractSPdataForAspectj(String inputDir, String outputDir) throws IOException {
		File[] files = new File(inputDir).listFiles();
		
		if (! new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}
		
		for (File file : files) {
			if (file.getName().startsWith(".")) {
				continue;
			}
			
			System.out.println("Working on ..." + file.getName());
			
			List<String> lines = FileHandler.readAllLines(file.getAbsolutePath());
			List<String> spLines = new ArrayList<String>();
			
			for (String line: lines) {
				String name = line.split(" ")[0].trim();
				String ef = line.split(" ")[2].trim();
				String ep = line.split(" ")[3].trim();
				String nf = line.split(" ")[4].trim();
				String np = line.split(" ")[5].trim();
				String spLine = name + " " + ef + " " + ep + " " + nf + " " + np;
				spLines.add(spLine);
			}
			
			FileHandler.writeListToFile(spLines, outputDir + "/" + file.getName()); 
		}
		
		System.out.println("Done");
		
	}
	
	public static void extractIR_Only_BugLocator_Data(String inputDir, String outputDir) throws IOException {
		
		File[] levels = new File(inputDir).listFiles();
		for (File level : levels) {
			if (level.getName().startsWith(".")) {
				continue;
			}
			
			File[] projects = new File(level.getAbsolutePath()).listFiles();
			
			for (File project : projects) {
				if (project.getName().startsWith(".")) {
					continue;
				}
				
				File[] files = new File(project.getAbsolutePath()).listFiles();
				for (File file : files) {
					if (file.getName().startsWith(".")) {
						continue;
					}
					
					System.out.println("Working on ..." + file.getName());
					
					List<String> newIRFormat = new ArrayList<String>();
					
					List<String> irLines = FileHandler.readAllLines(file.getAbsolutePath());
					
					for (String line : irLines) {
						String name = line.split(",")[1].trim();
						String score = line.split(",")[3].trim();
						newIRFormat.add(name + " " + "buglocator=" + score);
					}
					
					String outputFilePath = outputDir + "/" + level.getName() + "/" + project.getName();
					
					if (! new File(outputFilePath).exists()) {
						new File(outputFilePath).mkdirs();
					}
					
					FileHandler.writeListToFile(newIRFormat, outputFilePath + "/" + file.getName()); 
					
				}
			}
		}
		
	}
	
	public static void getBackScoreForBugLocator(String noScoreDir, String scoreDir, String outputDir) throws IOException {
		File[] projects = new File(noScoreDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".")) {
				continue;
			}
			File[] files = new File(project.getAbsolutePath()).listFiles();
			for (File file : files) {
				if (file.getName().startsWith(".")) {
					continue;
				}
				List<String> noScoreList = FileHandler.readAllLines(file.getAbsolutePath());
				List<String> scoreList = FileHandler.readAllLines(scoreDir + "/" + project.getName() + "/" + file.getName());
				
				List<String> addedScoreList = new ArrayList<String>();
				
				for (String line : noScoreList) {
					boolean found = false;
					for (String score : scoreList) {
						if (score.split(",")[1].endsWith(line) || line.endsWith(score.split(",")[1])) {
							String buglocatorScore = score.split(",")[3].trim();
							addedScoreList.add(line + " " + "buglocator=" + buglocatorScore);
							found = true;
							break;
						}
					}
					
					if (! found) {
						System.out.println("cant find a corresponding score. Something must go wrong");
						System.out.println(file.getName());
						System.exit(0);
					}
				}
				
				String outputFilePath = outputDir + "/" + project.getName();
				if (! new File(outputFilePath).exists()) {
					new File(outputFilePath).mkdirs();
				}
				
				FileHandler.writeListToFile(addedScoreList, outputFilePath + "/" + file.getName());
				
			}
		}
		
	}
	
	public static void extractIrScore(String inputIrScore, String outputIrScore) throws IOException {
		File[] levels = new File(inputIrScore).listFiles();
		for (File level : levels) {
			if (level.getName().startsWith(".")) {
				continue;
			}
			
			File[] projects = new File(level.getAbsolutePath()).listFiles();
			for (File project : projects) {
				if (project.getName().startsWith(".")) {
					continue;
				}
				String outputPath = outputIrScore + "/" + level.getName() + "/" + project.getName();
				if (! new File(outputPath).exists()) {
					new File(outputPath).mkdirs();
				}
				File[] files = new File(project.getAbsolutePath()).listFiles();
				for (File file : files) {
					if (file.getName().startsWith(".")) {
						continue;
					}
					
					List<String> irFullLines = FileHandler.readAllLines(file.getAbsolutePath());
					List<String> irShortenLines = new ArrayList<String>();
					
					for (String irFullLine : irFullLines) {
						String name = irFullLine.split(" ")[2].trim();
						String score = irFullLine.split(" ")[4].trim();
						irShortenLines.add(name + " " + "ir=" + score );
					}
					
					FileHandler.writeListToFile(irShortenLines, outputPath + "/" + file.getName());
				}
				
			}
		}
	}
	
	public static void countLOC(String inputDir, String outputFile) throws IOException {
		File[] projects = new File(inputDir).listFiles();
		
		List<String> result = new ArrayList<String>();
		
		int ingoredCount = 0;
		
		for (File project : projects) {
			if (project.getName().startsWith(".") || ! project.getName().equals("aspectj")) {
				continue;
			}
			
			System.out.println("Working on project: " + project.getName());
			
			int projectLineCode = 0;
			
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			for (File version : versions) {
				if (version.getName().startsWith(".")) {
					continue;
				}
				
				System.out.println("Working on version: " + project.getName() + ": " + version.getName());
				
				String codeDir = version.getAbsolutePath() + "/" + "pre-fix";
				
				if (project.getName().equals("lucene37")) {
					codeDir = codeDir + "/" + "lucene" + "/" ;
				}
				
				int offset = codeDir.length();
				FileSystem fs = new FileSystem();
				List<String> javaFiles = fs.getAllJavaFiles(codeDir, offset);
				
				int versionCodeLines = 0;
				
				for (String javaFile : javaFiles) {
					
//					if (javaFile.contains("StackMap")) {
//						System.out.println(++ingoredCount + ". ignore :" + javaFile);
//						continue;
//					}
					
					//versionCodeLines += FileHandler.readAllLines(javaFile).size();
					versionCodeLines += FileHandler.countLines(javaFile);
				}
				
				projectLineCode += versionCodeLines;
				
			}
			
			projectLineCode = projectLineCode/versions.length;
			
			result.add(project.getName() + " : " + projectLineCode);
			
			System.out.println(project.getName() + ": " + projectLineCode);
		}
		
		FileHandler.writeListToFile(result, outputFile);
		
		System.out.print("Done !");
	}
	
	public static void writeDataFileToLatexTable(String inputFile, String texOutputFile) throws IOException {
		List<String> rows = FileHandler.readAllLines(inputFile);
		List<String> outRows = new ArrayList<String>();
		
		//write data
		for (String row : rows) {
			outRows.add("\t\t\t" + row.replaceAll("\\s+", " & ") + "\\\\");
		}
		//write header
		outRows.add(0, "\\begin{table}[ht]");
		outRows.add(1, "\t \\scriptsize");
		outRows.add(2,  "\t \\begin{center}");
		outRows.add(3, "\t\t \\caption{}");
		outRows.add(4, "\t\t \\label{}");
		outRows.add(5,  "\t\t \\begin{tabular}" + "{|r|r|r|r|r|r|r|r|r|r|r|r|}");
		
		//write closing table tags
		
		outRows.add("\t\t \\end{tabular}");
		outRows.add("\t \\end{center}");
		outRows.add("\\end{table}");
		
		FileHandler.writeListToFile(outRows, texOutputFile);
		
	}
	
	public static class MapUtil
	{
	    public static <K, V extends Comparable<? super V>> Map<K, V> 
	        sortByValue( Map<K, V> map )
	    {
	        List<Map.Entry<K, V>> list =
	            new LinkedList<Map.Entry<K, V>>( map.entrySet() );
	        Collections.sort( list, new Comparator<Map.Entry<K, V>>()
	        {
	            public int compare( Map.Entry<K, V> o1, Map.Entry<K, V> o2 )
	            {
	                return (o1.getValue()).compareTo( o2.getValue() );
	            }
	        } );

	        Map<K, V> result = new LinkedHashMap<K, V>();
	        for (Map.Entry<K, V> entry : list)
	        {
	            result.put( entry.getKey(), entry.getValue() );
	        }
	        return result;
	    }
	}
	
	public static void getGroundtruthStatics(String outputFile) throws IOException {
		
		String pathToGroundTruth = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth";
		File[] projects = new File(pathToGroundTruth).listFiles();
		
		List<HashMap> methodStatics = new ArrayList<HashMap>();
		List<HashMap> classStatics = new ArrayList<HashMap>();
		
		for (File project : projects) {
			HashMap<String, Integer> method_stat_list = new HashMap<String, Integer>();
			HashMap<String, Integer> class_stat_list = new HashMap<String, Integer>();
			
			File[] bugs = new File(project.getAbsolutePath()).listFiles();
			for (File bug : bugs) {
				List<String> buggyLocations = FileHandler.readAllLines(bug.getAbsolutePath());
				
				String bugId = bug.getName().split("\\.")[0];
				
				method_stat_list.put(bugId, buggyLocations.size());
				
				List<String> classBuggyLocations = new ArrayList<String>();
				
				for (String location : buggyLocations) {
					if (location.contains("_")) {
						classBuggyLocations.add(location.split("_")[0]);
					} else {
						System.out.println("This buggy location does not contain underscore _ as expected: " + location + "=>" + bug.getAbsolutePath());
						System.exit(0);
					}
				}
				
				List<String> uniqueClassBuggyLocations = new ArrayList<String>(new HashSet<String>(classBuggyLocations));
				class_stat_list.put(bugId, uniqueClassBuggyLocations.size());
				
			}
			
			methodStatics.add((HashMap) MapUtil.sortByValue(method_stat_list));
			classStatics.add((HashMap) MapUtil.sortByValue(class_stat_list));
		}
		
//		System.out.println(methodStatics.size());
//		System.out.println(classStatics.size());
		
		List<String> finalMethodStats = new ArrayList<String>();
		List<String> finalClassStats = new ArrayList<String>();
		
		for (HashMap hs : methodStatics) {
			Iterator it = hs.entrySet().iterator();
			int sum = 0;
			int count = 0;
			
			int currentKeyValue = 1;
			int localCount = 0;
			
		    while (it.hasNext()) {
		        Map.Entry pair = (Map.Entry)it.next();
		        
		        if (pair.getValue().equals(currentKeyValue)) {
		        	localCount ++;
		        } else {
		        	System.out.println("-----------------------------");
		        	System.out.println(localCount + " bug report of " + currentKeyValue + " buggy locations");
		        	System.out.println("-----------------------------");
		        	currentKeyValue = (int) pair.getValue();
		        	localCount = 1;
		        }
		        
		        System.out.println(pair.getKey() + " = " + pair.getValue());
		        finalMethodStats.add(pair.getKey() + " = " + pair.getValue());
		        sum += Integer.parseInt(pair.getValue().toString());
		        count ++;
		        it.remove(); // avoids a ConcurrentModificationException
		    }
		  
		    System.out.println("-----------------------------");
        	System.out.println(localCount + " bug report of " + currentKeyValue + " buggy locations");
        	System.out.println("-----------------------------");
		    
		    finalMethodStats.add("-----------------------------");
		    finalMethodStats.add("No. BRs = " + count);
		    finalMethodStats.add("No. buggy Locations = " + sum);
		    finalMethodStats.add(String.format("Buggy Locations per BR = %.2f", (double) sum/count));
		    finalMethodStats.add("-----------------------------");
		}
		FileHandler.writeListToFile(finalMethodStats, "/Users/assist/Desktop/method-stat.txt");
		
		for (HashMap hs : classStatics) {
			Iterator it = hs.entrySet().iterator();
			int sum = 0;
			int count = 0;
		    while (it.hasNext()) {
		        Map.Entry pair = (Map.Entry)it.next();
		        System.out.println(pair.getKey() + " = " + pair.getValue());
		        finalClassStats.add(pair.getKey() + " = " + pair.getValue());
		        sum += Integer.parseInt(pair.getValue().toString());
		        count ++;
		        it.remove(); // avoids a ConcurrentModificationException
		    }
		    finalClassStats.add("-----------------------------");
		    finalClassStats.add("No. BRs = " + count);
		    finalClassStats.add("No. buggy Locations = " + sum);
		    finalClassStats.add(String.format("Buggy Locations per BR = %.2f", (double) sum/count));
		    finalClassStats.add("-----------------------------");
		}
		FileHandler.writeListToFile(finalClassStats, "/Users/assist/Desktop/class-stat.txt");
	}

	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		
		//extractSPdataForAspectj("/Users/assist/Desktop/tmp/ir-sp-method-ant53", "/Users/assist/Desktop/IRSPcombined/sp-data/method/ant53");
		
		//extractIR_Only_BugLocator_Data("/Users/assist/Desktop/IRSPcombined/ir-only-buglocator", "/Users/assist/Desktop/IRSPcombined/ir-data/buglocator");
		
		//getBackScoreForBugLocator("/Users/assist/Desktop/IRSPcombined/backup/ir-slicing-data-raw/buglocator/method", "/Volumes/osxrep/BugLocator/method/output-method-flatten", "/Users/assist/Desktop/IRSPcombined/ir-slicing-data/buglocator/method");
		
		//extractIrScore("/Users/assist/Desktop/IRSPcombined/backup-data/ir-cv-data/bluir", "/Users/assist/Desktop/IRSPcombined/ir-cv-data/bluir");
		
		//countLOC("/Volumes/osxrep/CODE-class", "/Volumes/osxrep/CODE-class/stat.txt");
		
		//extractIrScore("/Users/assist/Desktop/IRSPcombined/backup/ir-slicing-data-raw/buglocator", "/Users/assist/Desktop/IRSPcombined/ir-slicing-data/buglocator");

		//writeDataFileToLatexTable("/Users/assist/ase2016/tung-ase-2016/tables/ir-slicing-sp-buglocator-jaccard-method.txt", "/Users/assist/ase2016/tung-ase-2016/tables/table-slicing-sp-buglocator-jaccard-method.tex");
		
		getGroundtruthStatics("");
	}

}
