package td.utils;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class PomSearch {
	// Commit hash number patter
	public static String COMMIT_HASH = "\\s[a-z0-9]{40}";
	
	//check if a text is a commit number
	private static boolean matchCommitHash(String text){
		String patter = COMMIT_HASH;
		Pattern r = Pattern.compile(patter);
		Matcher m = r.matcher(text);
		return m.find();
	}
	
	private static List<String> readFile(String path){
		Charset charset = Charset.forName("UTF-8");
		try {
			return Files.readAllLines(Paths.get(path), charset);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	private static void writeListToFile(List<String> list, String filePath) throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(filePath)); 
		for(String line : list) {
		  writer.write(line);
		  writer.newLine();
		}
		writer.close();
	}
	
	/**
	 * parse in the git log file path, and specify
	 * the file name where you want to save the found commits
	 * and specify the content you want to search as well (e.g., pom.xml)
	 * @param pathToGitLogFile
	 * @param outputFile
	 * @param searchContent
	 * @throws IOException
	 */
	public static void searchPomFile (String pathToGitLogFile, String outputFile, String searchContent) throws IOException {
		List<String> lines = readFile(pathToGitLogFile);
		List<String> foundCommits = new ArrayList<String>();
		
		String latestCommit = "";
		for (String line : lines) {
			if (line.startsWith("commit") && matchCommitHash(line)) {
				latestCommit = line.split(" ")[1];
				System.out.println("Searching in commit: " + line.split(" ")[1].trim());
			} else {
				if (line.contains(searchContent)) {
					System.out.println(">>>" + searchContent + " found in this line: " + line);
					if (! foundCommits.contains(latestCommit)) {
						foundCommits.add(latestCommit);
					}
				}
			}
		}
		
		if (foundCommits.size() == 0) {
			System.out.println("No POM file is found");
		} else {
			writeListToFile(foundCommits, outputFile);
			System.out.println("Found commits have been saved to the file: " + outputFile);
		}
	}
	

	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		searchPomFile(args[0], args[1], args[2]);

	}

}
