package td.utils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class RankingsFilter {
	
	/**
	 * This filters out the ranks that
	 * are not in the coverage or slicing data
	 * @param motherRankFilePath
	 * @param filterFilePath
	 * @throws IOException
	 */
	public static ArrayList<String> filterRankings(String motherRankFilePath, String filterFilePath) throws IOException {
		List<String> originalRanks = FileHandler.readAllLines(motherRankFilePath);
		List<String> filterNames = FileHandler.readAllLines(filterFilePath);
		
		List<String> filterNamesCorrected = new ArrayList<String>();
		
		for (String line : filterNames) {
			String nLine = line;
			
//			if (line.contains("$")) {
//				nLine = line.split("$")[0];
//			} else {
//				nLine = line;
//			}
			
			if (nLine.contains("/")) {
				nLine = nLine.replaceAll("/", ".");
			}
			
			if (! nLine.endsWith(".java")) {
				nLine += ".java";
			}
			
			if (! nLine.startsWith(".")) {
				nLine = "." + nLine;
			}
			
			filterNamesCorrected.add(nLine);
		}
		
		Set<String> uniqueFilterNames = new HashSet<String>();
		
		uniqueFilterNames.addAll(filterNamesCorrected);
		
	
		ArrayList<String> resultingRanks = new ArrayList<String> ();
		
		for (String rank : originalRanks) {
			//String rankedName = rank.split(" ")[2].trim() + ".java";
			
			String rankedName = rank.split(",")[1].trim();
			
			if (!rankedName.startsWith(".")) {
				rankedName = "." + rankedName;
			}
			
			for (String filter : uniqueFilterNames) {
				//String filteringName = filter.split(" ")[2].trim() + ".java";
				if (rankedName.endsWith(filter) || filter.endsWith(rankedName)) {
					resultingRanks.add(rankedName);
					break;
				}
			}
		}
		
		printList(resultingRanks);
		
		System.out.println("------------------------------------");
		
		System.out.println("Size of resulting set: " + resultingRanks.size());
		System.out.println("Size of filter set: " + uniqueFilterNames.size());
		
		if (resultingRanks.size() != uniqueFilterNames.size()) {
			System.out.println("[Warnings]: The two set should have the same sizes");
		}
		
		return resultingRanks;
	}
	
	public static void printList(List<String> list) {
		for (String elem : list) {
			System.out.println(elem);
		}
	}
	
	public static void writeListToFile(ArrayList<String> list, String filePath) throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(filePath)); 
		for(String line : list) {
		  writer.write(line);
		  writer.newLine();
		}
		writer.close();
	}
	
	public static void doFilter(String motherFilePath, String filterFilePath, String outputFilePath) throws IOException{
		writeListToFile(filterRankings(motherFilePath, filterFilePath), outputFilePath);
	}
	
	public static void filterTheWholeProject(String motherSetPath, String filterSetPath, String resultSetPath) throws IOException {
		File[] motherFiles = new File(motherSetPath).listFiles();
		File[] filterFiles = new File(filterSetPath).listFiles();
		
		for (File file : motherFiles) {
			if (file.isFile() && ! file.getName().startsWith(".")) {
				String name = file.getName();
				File filterFile = new File(filterSetPath + "/" + name.split("\\.")[0]);
				if (filterFile.exists() && ! filterFile.isDirectory()) {
					String motherFileName = file.getAbsolutePath();
					String filterFileName = filterFile.getAbsolutePath();
					String outputFileName = resultSetPath + "/" + name;
					doFilter(motherFileName, filterFileName, outputFileName);
				} else {
					System.out.println("[Error] Something wrong at : " + name);
					System.out.println(file.getName() + "does not a corresponding file in Slicing or Coverage");
					//System.exit(0);
				}
			}
		}
		
	}

	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
	
//		String motherSet = "/Users/assist/dmz/gen-data/ir/method-ir/lucene37/all/results/post-fix";
//		String filterSet = "/Users/assist/dmz/gen-data/sIR_method_and_slicing/out/lucene37";
//		String resultSet = "/Users/assist/dmz/gen-data/sIR_method_and_slicing2/lucene37";
//		filterTheWholeProject(motherSet, filterSet, resultSet);
		
		filterTheWholeProject("/Volumes/osxrep/BugLocator/method/output-method-flatten/rhino", "/Users/assist/dmz/filters/slicing/method/rhino", "/Volumes/osxrep/BugLocator/method/output-method-flatten-slicing/rhino");
		
	}

}
