package td.utils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class TestsExecution {
	private String spectraFile;
	private List<Testcase> tests;
	private int totalFailedTests = 0;
	private int totalPassedTests = 0;
	private int totalTests = 0;
	
	/*
	 * Constructor for Rhino, Lucene, Ant
	 */
	TestsExecution (String faulttracerDataFile, String amlTestList) throws IOException {
		spectraFile = faulttracerDataFile;
		tests = getTestsFromFaulttracer(faulttracerDataFile);
		correctFailedTestsUsingAML(amlTestList);
		countTotalFPTests();
		
		if ((totalFailedTests == 0) || (totalTests == 0)) {
			System.out.println("This project version: " + faulttracerDataFile + "does not contain failed tests ==> invalid");
			System.exit(0);
		}
	}
	/*
	 * Constructor for AspectJ case
	 */
	TestsExecution (String cloverDataFile) {
		spectraFile = cloverDataFile;
		tests = getTestsFromClover(cloverDataFile);
		countTotalFPTests();
		failedTestExists();
		testExists();
	}
	
	private void failedTestExists(){
		if (totalFailedTests == 0) {
			System.out.println("This project version: " + spectraFile + "does not contain failed tests ==> invalid");
			//System.exit(0);
		}
	}
	
	private void testExists() {
		if (totalTests == 0) {
			System.out.println("This project version: " + spectraFile + "does not contain any tests ==> invalid");
			System.exit(0);
		}
	}
	
	public static List<String> extractItemsFromIRbasedRankedList(String filePathToList, String rankingMethod){ // rankingmethod = ir | bluir | buglocator
		List<String> rankedList = new ArrayList<String>();
		
		try {
			rankedList = FileHandler.readAllLines(filePathToList);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("Reading IR result list wrong at: " + filePathToList);
		}
		
		List<String> extractedList = new ArrayList<String>();
		int itemNamePos = 0, itemScorePos = 0;
		String dem = "";
		String irPrefix = "";
		switch (rankingMethod){
		case "ir":
			itemNamePos = 2;
			itemScorePos = 4;
			dem = " "; 
			irPrefix = "ir";
			break;
		case "bluir":
			itemNamePos = 2;
			itemScorePos = 4;
			dem = " ";
			irPrefix = "bluir";
			break;
		case "buglocator":
			itemNamePos = 1;
			itemScorePos = 3;
			dem = ",";
			irPrefix = "bl";
			break;
			default:
				System.out.println("[Error]. wrong ranking method");
				System.exit(0);
		}
		
		for (String item : rankedList){
			String itemName = item.split(dem)[itemNamePos];
			String itemScore = item.split(dem)[itemScorePos];
			String extractedItem = itemName + " " + irPrefix + "=" + itemScore;
			extractedList.add(extractedItem);
		}
		
		return extractedList;
	}
	
	public List<String> getSpectrumForListOfItems(List<String> items, String level) throws IOException {
		List<String> spectra = new ArrayList<String>();
		for (String item : items) {
			String itemName = item.split(" ")[0].trim();
			String itemScore = item.split(" ")[1].trim();
			Spectrum spectrum = getSpectrumOfAnElem(itemName, level);
			
			String spectrumLine = spectrum.getElemName() + " " + itemScore + " ef="+spectrum.getEf() + " ep=" + spectrum.getEp() + " nf=" + spectrum.getNf() + " np=" + spectrum.getNp();
			spectra.add(spectrumLine);
			//System.out.println(spectrumLine);
		}
		System.out.println(items.size());
		System.out.println(spectra.size());
		return spectra;
	}
	
	public List<String> getSpectrumForListOfItems2(String filePathToList, String rankingMethod, String level) throws IOException {
		
		List<String> items = extractItemsFromIRbasedRankedList(filePathToList, rankingMethod);
		
		int count = 0;
		
		List<String> spectra = new ArrayList<String>();
		for (String item : items) {
			String itemName = item.split(" ")[0].trim();
			String itemScore = item.split(" ")[1].trim();
			Spectrum spectrum = getSpectrumOfAnElem(itemName, level);
			
			String spectrumLine = spectrum.getElemName() + " " + itemScore + " ef="+spectrum.getEf() + " ep=" + spectrum.getEp() + " nf=" + spectrum.getNf() + " np=" + spectrum.getNp();
			spectra.add(spectrumLine);
			System.out.println(++count + ". " + spectrumLine);
		}
		System.out.println(items.size());
		System.out.println(spectra.size());
		System.out.println("done" + filePathToList);
		return spectra;
	}
	
	private void countTotalFPTests() {
		if (tests == null) {
			return;
		}
		
		for (Testcase t : tests){
			if (t.getResult().equals("passed")) {
				totalPassedTests += 1;
			} else {
				totalFailedTests += 1;
			}
		}
		totalTests = totalPassedTests + totalFailedTests;
		
		System.out.println("failed tests: " + totalFailedTests);
		System.out.println("passed tests: " + totalPassedTests);
		
		if (totalFailedTests < 1) {
			System.out.println("Something wrong. There must be at least 1 failed test");
			System.out.println(spectraFile);
		}
	}
	
	public List<String> getFailedTestsFromAML(String amlTestListFile) throws IOException{
		List<String> lines = FileHandler.readAllLines(amlTestListFile);
		List<String> failedTests = new ArrayList<String>();
		for (int i=1; i<lines.size(); i++){
			if (lines.get(i).split(",")[2].equals("failed")) {
				String failedTestName = lines.get(i).split(",")[0] + "." + lines.get(i).split(",")[1] ;
				failedTests.add(failedTestName);
			}
		}
		return failedTests;
	}
	
	public void correctFailedTestsUsingAML(String amlTestListFile) throws IOException{
		List<String> failedTests = getFailedTestsFromAML(amlTestListFile);
		boolean corrected = false;
		for (String t: failedTests) {
			for (Testcase tc : tests) {
				if (tc.getName().equals(t)) {
					tc.setResult("failed");
					System.out.println(tc.getName() + " has been corrected to failed");
					corrected = true;
				}
			}
		}
		
		if (! corrected){
			System.out.println("[error] no failed tests found" );
			System.out.println(amlTestListFile);
			//System.exit(0);
		}
			
	}
	
	/**
	 * Extract tests from colver data file, used for AspectJ case
	 * @param cloverDataFile
	 * @return
	 */
	private List<Testcase> getTestsFromClover(String cloverDataFile){
		List<Testcase> testcases = new ArrayList<Testcase>();
		try {
			List<String> lines = FileHandler.readAllLines(cloverDataFile);
			
			String testName = "";
			String testResult = "";
			List<String> coveredElems = null;
			
			int maxTests = 30; // limit the number of test is 100
			int countTests = 0;
			
			for (String line : lines){
				if (line.startsWith("<Test>")){
					Testcase test = new Testcase(testName, testResult, coveredElems);
					testcases.add(test);
					
					testName = line.split(":")[0].trim();
					testResult = line.split(":")[1].trim();
					coveredElems = new ArrayList<String>();
					
					countTests ++;
					
				} else {
					coveredElems.add(line);
				}
				
				if (countTests > maxTests) {
					break;
				}
				
			}
			
//			Testcase test = new Testcase(testName, testResult, coveredElems);
//			testcases.add(test);
			
			testcases.remove(0);
			
		} catch (Exception e) {
			System.out.println(e.getStackTrace());
			System.out.println("something wrong when processing file: " + cloverDataFile);
		}
		
		return testcases;
	}
	
	/**
	 * Extract tests from faulttracer data file
	 * @param faulttracerOutputFile
	 * @return
	 */
	private  List<Testcase> getTestsFromFaulttracer(String faulttracerOutputFile) {
		try {
			System.out.println("Working on : " + faulttracerOutputFile);
			List<String> lines = FileHandler.readAllLines(faulttracerOutputFile);
			
			List<Testcase> testcases = new ArrayList<Testcase>();

			String testName = "";
			String testResult = "";
			List<String> coveredElems = null;
			
			for (String line : lines) {
				if (line.contains("[java]")) {
					if (line.contains("<Test>")) {
						Testcase test = new Testcase(testName, testResult, coveredElems);	
						testcases.add(test);
						
						coveredElems = new ArrayList<String>();
						testName = line.split("<Test>")[1].trim();
						
						if (testName.endsWith("false")) {
							testName = testName.split("-false")[0].trim();
							testResult = "passed";
						} else {
							testName = testName.split("-true")[0].trim();
							testResult = "passed";
						}
		
						//System.out.println(testName);
						//System.out.println(testResult);
					} else if (! line.split("\\[java\\]")[1].trim().contains("WARN") && ! line.split("\\[java\\]")[1].trim().startsWith("Exception") && ! line.split("\\[java\\]")[1].trim().startsWith("at")) {
						//System.out.println(line);
						String coveredElem = line.trim().split(":")[0].split(" ")[1];
						//System.out.println(coveredElem);
						coveredElems.add(coveredElem);
					} else {
						System.out.println("line does not contain execution path");
						System.out.println(line);
					}
				}
			}
			
			Testcase test = new Testcase(testName, testResult, coveredElems);	
			testcases.add(test);
			
			testcases.remove(0);
			
			return testcases;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println(faulttracerOutputFile);
			return null;
		}
	}
	
	public Spectrum getSpectrumOfAnElem(String elemName, String level) {
		if (tests == null) return null;
		int ep = 0; //executed passing tests
		int ef = 0; //executed failing tests
		int nf = 0; //non-executed failing tests
		int np = 0; //non-executed passing tests
		
		for (Testcase t: tests) {
			List<String> coveredElems = t.getExecutedElement();
			//if (coveredElems.contains(elemName)) { // this condition is for method level
			//if (isElementCovered(coveredElems, elemName)) { // this is for class level
			
			if(isElementCovered(coveredElems, elemName, level)){
				//System.out.println("Test: " + t.getName() + " executed " + elemName);
				if (t.getResult().equals("failed")){
					ef += 1;
//					System.out.println(elemName);
//					System.out.println(t.getName());
//					System.out.println(t.getResult());
				} else {
					ep += 1 ;
				}
			}		
		}
		
		np = totalPassedTests - ep;
		nf = totalFailedTests - ef;
		
		Spectrum spectrum = new Spectrum();
		spectrum.setElemName(elemName);
		spectrum.setEf(ef);
		spectrum.setEp(ep);
		spectrum.setNf(nf);
		spectrum.setNp(np);
		
		return spectrum;

	}
	
	public static boolean isElementCovered(List<String> coveredElems, String elem, String level){		
		//for method case
		if (level.equals("method")){
			for (String e : coveredElems){
				if (elem.endsWith(e) || e.endsWith(elem)){
					return true;
				}
			}
		}
		// for class case
		if (! elem.endsWith(".java")) {
			elem += ".java";
		}
		
		for (String e : coveredElems){
			if (elem.endsWith(getClassName(e)) || getClassName(e).endsWith(elem)) {
				return true;
			}
			
		}
		
		return false;
	}
	
	
	public static String getClassName(String line){
		String[] parts = line.split("\\.");
		String className = "";
		for (String part : parts){
			if (Character.isUpperCase(part.charAt(0))) {
				className += part;
				break;
			} else {
				className += part + ".";
			}
		}
		if (className.contains("$")){
			className = className.split("\\$")[0];
		}
		
		if (! className.endsWith(".java")) {
			className += ".java";
		}
		return className;
	}
	
	public String getSpectraFile() {
		return spectraFile;
	}
	public void setSpectraFile(String spectraFile) {
		this.spectraFile = spectraFile;
	}
	public int getTotalFailedTests() {
		return totalFailedTests;
	}
	public void setTotalFailedTests(int totalFailedTests) {
		this.totalFailedTests = totalFailedTests;
	}
	public List<Testcase> getTests() {
		return tests;
	}
	public void setTests(List<Testcase> tests) {
		this.tests = tests;
	}
	public int getTotalPassedTests() {
		return totalPassedTests;
	}
	public void setTotalPassedTests(int totalPassedTests) {
		this.totalPassedTests = totalPassedTests;
	}
	public int getTotalTests() {
		return totalTests;
	}
	public void setTotalTests(int totalTests) {
		this.totalTests = totalTests;
	}

}
