#!/bin/sh

###--------input data----------------------------

dataset="$1" # project name
level="$2" # class or method level
indexing_type="$3"

field_path="/Users/assist/dmz/indri-params/fields"
stopword_path="/Users/assist/dmz/indri-params/stopwords"

### make sure all input arguments are correct ###

if [ $dataset == "ant53" ]
then
    echo "You are indexing : $dataset"
elif [ $dataset == "lucene37" ]
then
    echo "You are indexing project: $dataset"
elif [ $dataset == "rhino" ]
then
    echo "You are indexing project: $dataset"
elif [ $dataset == "aspectj" ]
then
    echo "You are indexing project: $dataset"
else
    echo "[error] You are entering incorrect argument. The program will exit..."
    exit -1
fi	     

if [ $level == "class" ]
then
    echo "Indexing at : $level level"
elif [ $level == "method" ]
then    
    echo "Indexing at: $level level"
else
    echo "[error] Incorrect argument! should be either class or method. The program will exit..."
    exit
fi

if [ $indexing_type == "raw" ]
then
    echo "Indexing without fields"
    corpuses="corpuses"
elif [ $indexing_type == "structured" ]
then
    echo "Indexing with fields"
    corpuses="structured-corpuses"
else
    echo "[error] incorrect indexing type"
    exit
fi


base="/Users/assist/dmz"
     
######## OUTPUT FOLDERS #########################
gen_dir="$base/gen-data/ir"

if [ ! -d "$gen_dir" ]
then
    echo "[error] Data generated folder ($gen_dir) does not exist."
    exit -1
else
    if [ ! -d "$gen_dir/$level-ir" ]
    then	
	echo "[error] Folder ($gen_dir/$level-ir) does not exist."
    fi
fi
    

JAVA_STRUCTURED_DIR="$gen_dir/$level-ir/$dataset/all/java"
INDEXES_DIR="$gen_dir/$level-ir/$dataset/all/$corpuses"

if [ ! -d "$JAVA_STRUCTURED_DIR" ]
then
    echo "[error] input extracted Java must be existed first"
    exit
fi

if [ ! -d "$INDEXES_DIR" ]
then    
    mkdir "$INDEXES_DIR"
else
    rm -rfv $INDEXES_DIR/*
fi

#################### START FROM HERE ################

BUGIDS=$(ls $JAVA_STRUCTURED_DIR)

for i in $BUGIDS
do
    mkdir "$INDEXES_DIR/$i"

    VERSIONS=$(ls $JAVA_STRUCTURED_DIR/$i)
    
    for j in $VERSIONS
    do
	echo "Start processing $i/$j ... [ok]"
	
	JVS_PATH="$JAVA_STRUCTURED_DIR/$i/$j"
	
	INDEX_PATH="$INDEXES_DIR/$i/$j"
	mkdir "$INDEX_PATH"
	if [ "$indexing_type" == "raw" ]
	then	    
	    IndriBuildIndex -corpus.path=$JVS_PATH -corpus.class=trectext -index=$INDEX_PATH
	elif [ "$indexing_type" == "structured" ]
	then
	    IndriBuildIndex $field_path -corpus.path=$JVS_PATH -corpus.class=trectext -index=$INDEX_PATH
	else
	    echo "[error] incorrect indexing type"
	    exit
	fi	
	
	echo "[input] ... [$INPUT_PATH]"
	echo "[output] ... [$JVS_PATH]"
	
	echo "Done processing $i/$j ... [ok]"
	
    done

done
